]> git.lyx.org Git - lyx.git/blobdiff - src/Row.h
Merge branch 'master' of git.lyx.org:lyx
[lyx.git] / src / Row.h
index 490aa6b749f5906c33442fce1253463e31989d22..f69eeca9f17ae9b83e4d24fafbafdbb105d1a133 100644 (file)
--- a/src/Row.h
+++ b/src/Row.h
@@ -29,18 +29,6 @@ namespace lyx {
 class DocIterator;
 class Inset;
 
-/**
- * FIXME: Change Row object to operate only on integers and not doubles.
- *
- * This use of double is only useful to distribute the extra
- * horizontal space between separators in justified text. If we do
- * integer arithmetic, then it is possible to have two groups of
- * separators, with size s or s+1. Then strings can be drawn without
- * cutting at separators in justfied text, as it is done in
- * non-justified text. This will improve performance.
- */
-
-
 /**
  * An on-screen row of text. A paragraph is broken into a RowList for
  * display. Each Row contains a tokenized description of the contents
@@ -58,8 +46,6 @@ public:
                 * correspond to any paragraph contents
                 */
                VIRTUAL,
-               // A stretchable space, basically
-               SEPARATOR,
                // An inset
                INSET,
                // Some spacing described by its width, not a string
@@ -76,7 +62,11 @@ public:
                          extra(0), font(f), change(ch), final(false) {}
 
                // Return total width of element, including separator overhead
-               double full_width() const { return dim.wid + extra; };
+               // FIXME: Cache this value or the number of separators?
+               double full_width() const { return dim.wid + extra * countSeparators(); }
+               // Return the number of separator in the element (only STRING type)
+               int countSeparators() const;
+
                /** Return position in pixels (from the left) of position
                 * \param i in the row element.
                 */
@@ -98,6 +88,13 @@ public:
                // Returns the position on right side of the element.
                pos_type right_pos() const;
 
+               //
+               bool isRTL() const { return font.isVisibleRightToLeft(); }
+               // This is true for virtual elements.
+               // Note that we do not use the type here. The two definitions
+               // should be equivalent
+               bool isVirtual() const { return pos == endpos; }
+
                // The kind of row element
                Type type;
                // position of the element in the paragraph
@@ -111,10 +108,10 @@ public:
                // Non-zero only if element is an inset
                Inset const * inset;
 
-               // Only non-null for separator elements
+               // Only non-null for justified rows
                double extra;
 
// Non-empty if element is a string or separator
                docstring str;
                //
                Font font;
@@ -148,6 +145,10 @@ public:
        void setSelectionAndMargins(DocIterator const & beg,
                DocIterator const & end) const;
 
+       ///
+       void pit(pit_type p) { pit_ = p; }
+       ///
+       pit_type pit() const { return pit_; }
        ///
        void pos(pos_type p) { pos_ = p; }
        ///
@@ -174,6 +175,16 @@ public:
        ///
        int descent() const { return dim_.des; }
 
+       /// The offset of the left-most cursor position on the row
+       int left_x() const;
+       /// The offset of the right-most cursor position on the row
+       int right_x() const;
+
+       // Return the number of separators in the row
+       int countSeparators() const;
+       // Set the extra spacing for every separator in STRING elements
+       void setSeparatorExtraWidth(double w);
+
        ///
        void add(pos_type pos, Inset const * ins, Dimension const & dim,
                 Font const & f, Change const & ch);
@@ -184,9 +195,6 @@ public:
        void addVirtual(pos_type pos, docstring const & s,
                        Font const & f, Change const & ch);
        ///
-       void addSeparator(pos_type pos, char_type const c,
-                         Font const & f, Change const & ch);
-       ///
        void addSpace(pos_type pos, int width, Font const & f, Change const & ch);
 
        ///
@@ -224,8 +232,9 @@ public:
         * remains is a large word, cut it to \param width.
         * \param body_pos minimum amount of text to keep.
         * \param width maximum width of the row
+        * \return true if the row has been shortened.
         */
-       void shortenIfNeeded(pos_type const body_pos, int const width);
+       bool shortenIfNeeded(pos_type const body_pos, int const width);
 
        /**
         * If last element of the row is a string, compute its width
@@ -241,7 +250,7 @@ public:
 
        friend std::ostream & operator<<(std::ostream & os, Row const & row);
 
-       /// width of a separator (i.e. space)
+       /// additional width for separators in justified rows (i.e. space)
        double separator;
        /// width of hfills in the label
        double label_hfill;
@@ -282,6 +291,8 @@ private:
        mutable bool changed_;
        /// CRC of row contents.
        mutable size_type crc_;
+       /// Index of the paragraph that contains this row
+       pit_type pit_;
        /// first pos covered by this row
        pos_type pos_;
        /// one behind last pos covered by this row
@@ -111,10 +108,10 @@ public:
                double extra;
 
-               // Non-empty if element is a string or separator
+               // Non-empty if element is a string or is virtual
                docstring str;
                //